#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""PostgreSQL in Thread v0.07
From iamvic on www.cyberforum.ru
Restrictions: Python 3.x, PyQt and no more.
"""

import copy

from PyQt5.QtWidgets import QApplication
from PyQt5.QtWidgets import QDialog, QMessageBox
from PyQt5.QtWidgets import QLineEdit, QPushButton
from PyQt5.QtWidgets import QFormLayout, QGroupBox
from PyQt5.QtWidgets import QHBoxLayout, QVBoxLayout
from PyQt5.QtSql import QSqlDatabase, QSqlQuery

from func_store import db_error_msg, query_error_msg


class Login_window(QDialog):
    '''Диалог запроса параметров соединения
    '''
    def __init__(self, parent=None,
                 connect_name='qt_sql_default_connection',
                 connect_parm=['localhost','5432','postgres',
                               'postgres','postgres']):
        super(Login_window, self).__init__(parent)
        self.setObjectName(self.metaObject().className())
        self.connect_name = connect_name
        self.connect_parm = connect_parm
        self.setWindowTitle('Параметры соединения')

        QSqlDatabase.database(self.connect_name, open=False).close()

        self.initUI()

    def initUI(self):
        self.host = QLineEdit(self.connect_parm[0], self)
        self.port = QLineEdit(self.connect_parm[1], self)
        self.port.setInputMask('99999')
        self.dbname = QLineEdit(self.connect_parm[2], self)
        self.user = QLineEdit(self.connect_parm[3], self)
        self.passwd = QLineEdit(self.connect_parm[4], self)
        self.passwd.setEchoMode(QLineEdit.Password)

        self.login_form = QFormLayout(self)
        self.login_form.addRow('Хост', self.host)
        self.login_form.addRow('Порт', self.port)
        self.login_form.addRow('База данных', self.dbname)
        self.login_form.addRow('Пользователь', self.user)
        self.login_form.addRow('Пароль', self.passwd)

        self.makeButton = QPushButton('Открыть', self)
        self.makeButton.clicked.connect(self.to_open)

        self.undoButton = QPushButton('Отменить', self)
        self.undoButton.clicked.connect(self.to_undo)

        gbox = QGroupBox('Параметры соединения', self)
        gbox.setLayout(self.login_form)

        hbox = QHBoxLayout(None)
        hbox.addWidget(self.makeButton)
        hbox.addStretch(1)
        hbox.addWidget(self.undoButton)

        mainLayout = QVBoxLayout(None)
        mainLayout.setSpacing(6)
        mainLayout.addWidget(gbox)
        mainLayout.addStretch(1)
        mainLayout.addLayout(hbox)

        self.setLayout(mainLayout)

    def to_undo(self):
        self.reject()

    def to_open(self):
        db = QSqlDatabase.database(self.connect_name, open=False)
        if db.isOpen():
            db.close()
        db.setHostName(self.host.text().strip())
        db.setPort(int(self.port.text().strip()))
        db.setDatabaseName(self.dbname.text().strip())
        db.setUserName(self.user.text().strip())
        db.setPassword(self.passwd.text())

        if not db.open():
            db_error_msg(self, db)
        else:
            self.connect_parm = [
                self.host.text().strip(),
                self.port.text().strip(),
                self.dbname.text().strip(),
                self.user.text().strip(),
                self.passwd.text()
                ]

            query = QSqlQuery(db)
            if not query.exec(
                "SET application_name TO '{!s} (main thread {!s})';".format(
                    QApplication.applicationName(), hex(id(self.thread())))):
                # хоть какая-то идентификация для админа БД.
                # QSqlQuery.prepare() с SET не работает
                # (видимо, о нём не знает).

                query_error_msg(self, query)

        self.accept()

    def login_to_db(
            client,
            connect_name='qt_sql_default_connection',
            connect_parm=['localhost','5432','postgres',
                          'postgres','postgres']):
        '''для наружного употребления, подобно Static Protected Members.
        Способ употребления:
        rc = Login_window.login_to_db(parent, connect_name, connect_parm)
        '''

        ex = Login_window(client, connect_name, connect_parm)
        rc = ex.exec_()
        if rc == QDialog.Accepted:
            rc_value = copy.deepcopy(ex.connect_parm)
        else:
            rc_value = None
        del ex
        return rc_value

