#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""PostgreSQL in Thread v0.07
From iamvic on www.cyberforum.ru
Restrictions: Python 3.x, PyQt and no more.
"""

import os, platform, sys

from PyQt5.QtWidgets import QApplication
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtSql import QSqlDatabase

ROOT32_REGKEY = 'SOFTWARE\\Wow6432Node\\'
ROOT_REGKEY = 'SOFTWARE\\'
PGSQL_KEY = 'PostgreSQL Global Development Group\\PostgreSQL'
PGSQL_LOCATION = 'Location'

if __name__ == '__main__':
    app = QApplication(sys.argv)
    app.setApplicationName('QPSQL_IN_QTHREAD')

    if sys.platform == 'win32':
        # под Windows выполним автонастройку на подходяшего клиента,
        # что избавляет от необходимости вручную прописывать
        # пути в переменных окружения
        import winreg
        _path = os.environ['PATH']

        # определяемся с разрядностью для Windows
        if platform.machine() == 'x86':
            # Windows/32, Python/32 => PostgreSQL/32
            _key = ROOT_REGKEY

        elif sys.maxsize > 0x100000000:
            # Windows/64, Python/64 => PostgreSQL/64
            _key = ROOT_REGKEY

        else:
            # Windows/64, Python/32 => PostgreSQL/32
            _key = ROOT32_REGKEY

        _location = ''
        # путь к PostgreSQL тащим из системного реестра Windows
        try:
            with winreg.OpenKey(
                winreg.HKEY_LOCAL_MACHINE, _key + PGSQL_KEY) as _reg_key:

                for i in range(16):
                    _key_val = winreg.EnumValue(_reg_key, i)
                    if _key_val[0] == PGSQL_LOCATION:
                        _location = _key_val[1]
                        break
        except:
            # поскольку есть умельцы, которые вместо полной установки
            # PostgreSQL пользуют вырезанную оттуда клиентскую часть,
            # то исключение генерить не будем и пусть им повезёт
            pass

        if len(_location) > 0:
            # добавляем пути в переменную окружения PATH
            os.environ['PATH'] = '{!s}\\bin;{!s}\\lib;{!s}'.format(
                _location, _location, _path)

    db = QSqlDatabase.addDatabase('QPSQL')

    if not (db.isValid()):
        # соединение не создано
        if sys.maxsize > 0x100000000:
            _bit = '(64bit)'
        else:
            _bit = '(32bit)'

        QMessageBox.critical(
            None, 'О Ш И Б К А !!!',
            'Запуск приложения невозможен.\n\n'
            'Для запуска приложения был использован Python {!s} v.{!s}.\n'
            'Для работы приложения под {!s} требуется:\n'
            '- установить PostgreSQL {!s}.\n'
            'Закройте приложение, проверьте все ли пакеты'
            ' установлены, установите недостающие и повторите'
            ' запуск.'.format(_bit, sys.version.split()[0],
                              platform.system(), _bit))

        sys.exit(1)

    from main_window import Main_window
    # добрались сюда => PostgreSQL установлен, доступ к нему настроен
    # соединение по умолчанию создано

    m_win = Main_window()
    m_win.show()

    sys.exit(app.exec_())
