#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""PostgreSQL in Thread v0.07
From iamvic on www.cyberforum.ru
Restrictions: Python 3.x, PyQt and no more.
"""

import sys

from PyQt5.QtCore import pyqtSignal, QTimer
from PyQt5.QtGui import QFont
from PyQt5.QtWidgets import QAction, QApplication
from PyQt5.QtWidgets import QMainWindow, QMessageBox
from PyQt5.QtWidgets import QTextEdit
from PyQt5.QtSql import QSqlDatabase

from login_window import Login_window
from thread_kit import Thread_postgresql

class Main_window(QMainWindow):
    '''Главное окно приложения
    '''
    dont_repeate_closing = False
    another_threads_closed = pyqtSignal()
    is_run = False

    def __init__(self, parent=None):
        super(Main_window, self).__init__(parent)
        self.setObjectName(self.metaObject().className())

        self.default_connection_name = QSqlDatabase.connectionNames()[0]
        # на момент создания главного окна уже существует
        # именно одно соединение и это соединение по умолчанию
        # (см. main.py)

        self.current_connection_parm = [
            'localhost','5432','postgres','postgres','postgres']
        self.bad_connection_parm = [
            'localhost',0,'postgres','postgres','postgres']

        self.initUI()

    def initUI(self):
        '''рисуем интерфейс
        '''
        self.statusBar()

        self.thread_action = QAction(
            'Старт', self, shortcut='Ctrl+S',
            statusTip=('Запустить дополнительный поток'
                       ' и выполнить в нём тест'),
            triggered=self.to_start_thread, enabled=True)

        self.login_action = QAction(
            'Регистрация', self, shortcut='Ctrl+L',
            statusTip='Получить доступ к базе данных',
            triggered=self.to_login, enabled=True)

        self.bad_parm_action = QAction(
            'Ошибка: параметр', self, shortcut='Ctrl+B',
            statusTip=('Запустить дополнительный поток'
                       'с некондиционными параметрами соединения'),
            triggered=self.to_bad_parm, enabled=True)

        self.run_action = QAction(
            'Ошибка: запуск', self, shortcut='Ctrl+R',
            statusTip=('Запустить дополнительный поток'
                       ' методом run() вместо start()'),
            triggered=self.to_run_thread, enabled=True)

        self.interrupt_action = QAction(
            'Быстрый выход', self, shortcut='Ctrl+T',
            statusTip=('Завершить работу с запросом'
                       ' на прерывание выполнения потоков'),
            triggered=self.to_interrupt, enabled=True)

        self.exit_action = QAction(
            'Выход с ожиданием', self, shortcut='Ctrl+Q',
            statusTip=('Завершить работу с ожиданием'
                       ' завершения потоков'),
            triggered=self.to_close, enabled=True)

        self.task_menu = self.menuBar().addMenu('Файл')
        self.task_menu.addAction(self.login_action)
        self.task_menu.addAction(self.thread_action)
        self.task_menu.addSeparator()
        self.task_menu.addAction(self.bad_parm_action)
        self.task_menu.addAction(self.run_action)
        self.task_menu.addSeparator()
        self.task_menu.addAction(self.interrupt_action)
        self.task_menu.addAction(self.exit_action)
        self.to_switch_state()

        self.protocol = QTextEdit()
        font = self.protocol.font()
        font.setWeight(QFont.Bold)
        self.protocol.setFont(font)
        self.protocol.setReadOnly(True)
        self.setCentralWidget(self.protocol)
        self.to_refresh_protocol()



    def to_login(self):
        '''устанавливаем соединение с БД
        '''
        QSqlDatabase.database(
            self.default_connection_name, open=False).close()
        # на всякий случай закрываем текущее соединение

        self.current_connection_parm[4]=''
        # очищаем пароль в сохранённых параметрах

        rc = Login_window.login_to_db(self,
                                      self.default_connection_name,
                                      self.current_connection_parm)
        if rc is not None:
            # соединение успешно установлено
            self.current_connection_parm = rc
            # сохраняем параметры вновь установленного соединения

        self.to_switch_state()
        # обновляем состояние меню

    def to_start_thread(self):
        '''стартуем дополнительный поток
        '''
        abettor = Thread_postgresql(self)
        if abettor.set_connection_parm(self.current_connection_parm):
            abettor.connection_created.connect(self.to_refresh_protocol)
            # abettor.started.connect(self.to_refresh_protocol)
            # интересная особенность - под Linux хватало сигнала started,
            # а под Windows пришлось свой сигнал городить,
            # иначе соединения не успевали попадать в протокол

            abettor.finished.connect(abettor.deleteLater)
            abettor.finished.connect(self.to_refresh_protocol)
            abettor.invalid_using.connect(self.to_invalid_using)
            abettor.duplicated_thread.connect(self.to_duplicated_thread)
            if self.is_run:
                # выбран пункт меню (Файл -> Ошибка: запуск)
                abettor.run()
                del abettor
            else:
                # выбран пункт меню (Файл -> Старт)
                abettor.start()
        else:
            del abettor
            QMessageBox.warning(
                self, 'П Р Е Д У П Р Е Ж Д Е Н И Е !!!',
                'Нарушение структуры параметров, передаваемых'
                ' в дополнительный поток.\n'
                'Дополнительный поток не создан.')

    def to_bad_parm(self):
        '''стартуем дополнительный поток c некондиционными параметрами
        '''
        store_parm = self.current_connection_parm
        self.current_connection_parm = self.bad_connection_parm
        self.to_start_thread()
        self.current_connection_parm = store_parm

    def to_run_thread(self):
        '''выполняем дополнительный поток методом run()
        '''
        self.is_run = True
        self.to_start_thread()
        self.is_run = False

    def to_interrupt(self):
        '''закрываем приложение с запросом на прерывание выполнения потоков
        '''
        for t in self.children():
            if type(t) == Thread_postgresql:
                t.requestInterruption()

        if self.exit_action.isEnabled():
            self.to_close()

    def to_close(self):
        '''закрываем приложение с ожиданием завершения потоков
        '''
        self.dont_repeate_closing = True
        self.login_action.setEnabled(False)
        self.thread_action.setEnabled(False)
        self.bad_parm_action.setEnabled(False)
        self.run_action.setEnabled(False)
        self.exit_action.setEnabled(False)
        QSqlDatabase.database(
            self.default_connection_name, open=False).close()

        if len(QSqlDatabase.connectionNames()) > 1:
           # есть дополнительные потоки ==> ожидаем завершения
           self.to_waiting()
        else:
           self.close()



    def to_switch_state(self):
        '''обновляем состояние меню
        '''
        state = QSqlDatabase.database(
            self.default_connection_name, open=False).isOpen()
        self.thread_action.setEnabled(state)
        self.bad_parm_action.setEnabled(state)
        self.run_action.setEnabled(state)

    def to_refresh_protocol(self):
        '''обновляем протокол и рассылаем сигналы
        '''
        text = ''
        for s in QSqlDatabase.connectionNames():
            text = text + s + '\n'
        self.protocol.setText(text)

        x = len(QSqlDatabase.connectionNames())
        if x <= 1:
            self.another_threads_closed.emit()

    def to_invalid_using(self):
        QMessageBox.warning(
            self, 'П Р Е Д У П Р Е Ж Д Е Н И Е !!!',
            'Для запуска дополнительного потока следует использовать'
            ' метод start(), а не run().\n'
            'Дополнительный поток не работоспособен.')

    def to_duplicated_thread(self):
        QMessageBox.warning(
            self, 'П Р Е Д У П Р Е Ж Д Е Н И Е !!!',
            'Дубль идентификатора дополнительного потока.\n'
            'Невозможно создать соединение с уникальным именем.')

    def to_waiting(self):
        '''ожидаем завершения дополнительных потоков,
        можно было и QProgressBar организовать, но тут и так
        хорошо видно как соединения пропадают из протокола
        '''
        self.statusBar().showMessage(
            'Ожидание завершения дополнительных потоков')
        self.another_threads_closed.connect(self.close)



    def closeEvent(self, e):
        '''события от внешних источников игнорируем,
        организуем выполнение процедуры закрытия приложения
        вне обработчика событий
        '''
        if self.sender() is None:
            e.ignore()
            if not self.dont_repeate_closing:
                self.dont_repeate_closing = True
                QTimer.singleShot(200, self.to_close)
                # QTimer вызовет процедуру закрытия
                # уже после выхода из обработчика событий
        else:
            e.accept()
